"""Banned Tracks per Zone Model"""

from sqlalchemy import UUID, Integer
from sqlalchemy.orm import Mapped, mapped_column
from artemis_model.base import AuditMixin, TimeStampMixin, CustomSyncBase, CustomBase


class BannedTracksMixin(TimeStampMixin, AuditMixin):
    """Banned Tracks per Zone Model"""

    zone_id: Mapped[int] = mapped_column(Integer, primary_key=True)
    track_id: Mapped[UUID] = mapped_column(UUID(as_uuid=True), primary_key=True)


class BannedTracksSync(CustomSyncBase, BannedTracksMixin):
    """Banned Tracks per Zone Model"""

    pass


class BannedTracks(CustomBase, BannedTracksMixin):
    """Banned Tracks per Zone Model"""

    pass



import uuid
from datetime import datetime

from sqlalchemy import Computed, ForeignKey, ARRAY, Integer
from sqlalchemy.ext.associationproxy import AssociationProxy, association_proxy
from sqlalchemy.orm import Mapped, mapped_column, relationship

from artemis_model.base import CustomBase, CustomSyncBase, TSVector, TimeStampMixin, AuditMixin
from sqlalchemy.ext.declarative import declared_attr


class DjSetGenreAssocMixin(TimeStampMixin):
    dj_set_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("dj_set.id"), primary_key=True, nullable=False
    )
    genre_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("genre.id"), primary_key=True, nullable=False
    )
    weight: Mapped[int] = mapped_column(nullable=False, default=0)

    @declared_attr
    def genre(cls) -> Mapped["Genre"]:
        return relationship(back_populates="dj_set_genre_associations")


class DjSetGenreAssocSync(CustomSyncBase, DjSetGenreAssocMixin):
    pass


class DjSetGenreAssoc(CustomBase, DjSetGenreAssocMixin):
    pass


class DjSetMixin(TimeStampMixin, AuditMixin):
    id: Mapped[int] = mapped_column(primary_key=True, autoincrement=True)
    name: Mapped[str] = mapped_column(nullable=False, unique=True)
    organization_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("organization.id"), nullable=True, index=True
    )
    entry_date: Mapped[datetime] = mapped_column(nullable=False, default=datetime.utcnow)
    cover_image: Mapped[str] = mapped_column(nullable=True)
    disabled: Mapped[bool] = mapped_column(nullable=False, default=False)
    legacy_id: Mapped[str] = mapped_column(nullable=True)
    description: Mapped[str | None] = mapped_column(nullable=True)
    mood: Mapped[list[int]] = mapped_column(ARRAY(Integer), nullable=False, default=[0, 0])
    decades: Mapped[list[int] | None] = mapped_column(ARRAY(Integer), nullable=True)
    total_duration: Mapped[int] = mapped_column(nullable=False, default=0)

    name_tsv = mapped_column(
        TSVector(),
        Computed("to_tsvector('english', name)", persisted=True),
    )

    @declared_attr
    def organization(cls) -> Mapped["Organization"]:
        return relationship(back_populates="dj_sets")

    @declared_attr
    def genre_ids(cls) -> AssociationProxy[list["Genre"] | None]:
        return association_proxy(
            "dj_set_genre_associations",
            "genre",
            creator=lambda genre: DjSetGenreAssoc(
                genre_id=genre.get("id"), weight=genre.get("weight", 0)
            ),
        )

    @declared_attr
    def dj_set_genre_associations(cls) -> Mapped[list["DjSetGenreAssoc"]]:
        return relationship(cascade="all, delete-orphan")

    @declared_attr
    def dj_set_track_associations(cls) -> Mapped[list["DjSetTrackAssoc"]]:
        return relationship(cascade="all, delete-orphan")

    @declared_attr
    def genres(cls) -> Mapped[list["Genre"]]:
        return relationship(secondary="dj_set_genre_assoc", lazy="joined", viewonly=True)


@declared_attr
def tracks(cls) -> Mapped[list["Track"]]:
    return relationship(secondary="dj_set_track_assoc", viewonly=True)


class DjSetSync(CustomSyncBase, DjSetMixin):
    pass


class DjSet(CustomBase, DjSetMixin):
    pass


class DjSetTrackAssocMixin(TimeStampMixin):
    dj_set_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("dj_set.id"), primary_key=True, nullable=False
    )
    track_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("track.id"), primary_key=True, nullable=False
    )

    @declared_attr
    def track(cls) -> Mapped["Track"]:
        return relationship(back_populates="dj_set_track_associations")


class DjSetTrackAssocSync(CustomSyncBase, DjSetTrackAssocMixin):
    pass


class DjSetTrackAssoc(CustomBase, DjSetTrackAssocMixin):
    pass



"""Artist models"""

from datetime import datetime

from sqlalchemy import Computed, func, literal, text
from sqlalchemy.orm import Mapped, mapped_column, relationship

from artemis_model.base import CustomBase, CustomSyncBase, TSVector, TimeStampMixin

from sqlalchemy.ext.declarative import declared_attr


def to_tsvector_ix(*columns):
    s = " || ' ' || ".join(columns)
    return func.to_tsvector(literal("english"), text(s))


class ArtistMixin(TimeStampMixin):
    id: Mapped[int] = mapped_column(primary_key=True, autoincrement=True)
    name: Mapped[str] = mapped_column(nullable=False)
    entry_date: Mapped[datetime] = mapped_column(nullable=False, default=datetime.utcnow)
    disabled: Mapped[bool] = mapped_column(nullable=False, default=False)
    is_internal: Mapped[bool] = mapped_column(nullable=False, default=False)
    legacy_id: Mapped[str] = mapped_column(nullable=False)

    name_tsv = mapped_column(
        TSVector(),
        Computed("to_tsvector('english', name)", persisted=True),
    )

    # __table_args__ = (
    #     Index("fts_ix_artist_name_tsv", to_tsvector_ix("name"), postgresql_using="gin"),
    # )

    @declared_attr
    def albums(cls) -> Mapped[list["Album"]]:
        return relationship(secondary="album_artist_assoc", back_populates="artists")

    @declared_attr
    def tracks(cls) -> Mapped[list["Track"]]:
        return relationship("Track", back_populates="artist", cascade="all, delete-orphan")


class ArtistSync(CustomSyncBase, ArtistMixin):
    pass


class Artist(CustomBase, ArtistMixin):
    pass



"""Zone Activity Model"""

from enum import StrEnum
from sqlalchemy import Integer, String, JSON, Index
from sqlalchemy.orm import Mapped, mapped_column
from artemis_model.base import AuditMixin, TimeStampMixin, CustomSyncBase, CustomBase


class ZoneActivityType(StrEnum):
    """Zone activity type"""

    CONNECTED = "connected"
    CONNECTION_MODE_CHANGED = "connection_mode_changed"
    DISCONNECTED = "disconnected"
    PLAYER_PAUSED = "player_paused"
    PLAYER_RESUMED = "player_resumed"
    TRACK_SKIPPED = "track_skipped"
    TRACK_LIKED = "track_liked"
    TRACK_BANNED = "track_banned"
    TRACK_UNBANNED = "track_unbanned"
    PLAYLIST_PUSHED = "playlist_pushed"
    PLAYLIST_EXPIRED = "playlist_expired"
    ACTIVE_PLAYLIST_CHANGED = "active_playlist_changed"


class PlayerActivityMixin(TimeStampMixin, AuditMixin):
    """User activity log, sorted by created_at DESC for retrieval"""

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    player_id: Mapped[int] = mapped_column(Integer, nullable=False)
    activity_type: Mapped[str] = mapped_column(String, nullable=False)
    activity_data: Mapped[dict] = mapped_column(JSON, nullable=True)


Index("idx_player_activity_player_created", "player_id", PlayerActivityMixin.created_at.desc())
Index("idx_player_activity_created", PlayerActivityMixin.created_at.desc())

class PlayerActivitySync(CustomSyncBase, PlayerActivityMixin):
    """Sync model for Player Activity"""

    pass


class PlayerActivity(CustomBase, PlayerActivityMixin):
    """Base model for Player Activity"""

    pass



import uuid
from datetime import date, datetime

from sqlalchemy import Computed, ForeignKey, func, literal, text
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy.ext.associationproxy import AssociationProxy, association_proxy
from sqlalchemy.orm import Mapped, mapped_column, relationship

from artemis_model.base import CustomBase, CustomSyncBase, TSVector, TimeStampMixin
from sqlalchemy.ext.declarative import declared_attr


def to_tsvector_ix(*columns):
    s = " || ' ' || ".join(columns)
    return func.to_tsvector(literal("english"), text(s))


class FavoritePlaylistMixin:
    playlist_id: Mapped[int] = mapped_column(ForeignKey("playlist.id"), primary_key=True)
    user_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("user.id"), primary_key=True)


class FavoritePlaylistSync(CustomSyncBase, FavoritePlaylistMixin):
    pass


class FavoritePlaylist(CustomBase, FavoritePlaylistMixin):
    pass


class PlaylistMixin(TimeStampMixin):
    id: Mapped[int] = mapped_column(primary_key=True, autoincrement=True)
    name: Mapped[str] = mapped_column(nullable=False, unique=True)
    entry_date: Mapped[datetime] = mapped_column(nullable=False, default=datetime.utcnow)
    disabled: Mapped[bool] = mapped_column(nullable=False, default=False)
    legacy_id: Mapped[str] = mapped_column(nullable=True)
    is_published: Mapped[bool] = mapped_column(default=False)  # legacy isTest
    description: Mapped[str | None] = mapped_column(nullable=True)
    cover_image: Mapped[str | None] = mapped_column(nullable=True)
    is_ordered: Mapped[bool] = mapped_column(default=False)
    legacy_prepared_by: Mapped[str | None] = mapped_column(nullable=True)
    notes: Mapped[str | None] = mapped_column(nullable=True)
    published_at: Mapped[date | None] = mapped_column(nullable=True)
    is_private: Mapped[bool] = mapped_column(default=False)
    total_duration: Mapped[int] = mapped_column(default=0)

    name_tsv = mapped_column(
        TSVector(),
        Computed("to_tsvector('english', name)", persisted=True),
    )

    @declared_attr
    def favorite_playlist_associations(cls):
        return relationship(
            "FavoritePlaylist",
            primaryjoin="and_(FavoritePlaylist.playlist_id == Playlist.id)",
            viewonly=True,
            lazy="select",
        )

    @hybrid_property
    def is_favorite(self):
        return bool(self.favorite_playlist_associations)

    @declared_attr
    def category_playlist_associations(cls) -> Mapped[list["PlaylistCategoryAssoc"]]:
        return relationship(cascade="all, delete-orphan")

    @declared_attr
    def track_playlist_associations(cls) -> Mapped[list["PlaylistTrackAssoc"]]:
        return relationship(cascade="all, delete-orphan")

    @declared_attr
    def organization_playlist_associations(cls) -> Mapped[list["PlaylistOrganizationAssoc"]]:
        return relationship(cascade="all, delete-orphan")

    @declared_attr
    def organization_ids(cls) -> AssociationProxy[list["Organization"] | None]:
        return association_proxy(
            "organization_playlist_associations",
            "organization",
            creator=lambda o: PlaylistOrganizationAssoc(organization_id=o),
        )

    @declared_attr
    def category_ids(cls) -> AssociationProxy[list["Category"] | None]:
        return association_proxy(
            "category_playlist_associations",
            "category",
            creator=lambda c: PlaylistCategoryAssoc(category_id=c),
        )

    @declared_attr
    def track_ids(cls) -> AssociationProxy[list["Track"] | None]:
        return association_proxy(
            "track_playlist_associations",
            "track",
            creator=lambda t: PlaylistTrackAssoc(track_id=t),
        )

    @declared_attr
    def tracks(cls) -> Mapped[list["Track"]]:
        return relationship(secondary="playlist_track_assoc", viewonly=True)

    @declared_attr
    def categories(cls) -> Mapped[list["Category"]]:
        return relationship(secondary="playlist_category_assoc", viewonly=True)

    # __table_args__ = (
    #     Index(
    #         "fts_ix_playlist_name_tsv",
    #         to_tsvector_ix("name"),
    #         postgresql_using="gin",
    #     ),
    # )


class PlaylistSync(CustomSyncBase, PlaylistMixin):
    pass


class Playlist(CustomBase, PlaylistMixin):
    pass


class PlaylistOrganizationAssocMixin(TimeStampMixin):
    playlist_id: Mapped[int] = mapped_column(
        ForeignKey("playlist.id"), primary_key=True, nullable=False
    )
    organization_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("organization.id"), primary_key=True, nullable=False
    )


class PlaylistOrganizationkAssocSync(CustomSyncBase, PlaylistOrganizationAssocMixin):
    pass


class PlaylistOrganizationAssoc(CustomBase, PlaylistOrganizationAssocMixin):
    pass


class PlaylistTrackAssocMixin(TimeStampMixin):
    playlist_id: Mapped[int] = mapped_column(
        ForeignKey("playlist.id"), primary_key=True, nullable=False
    )
    track_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("track.id"), primary_key=True, nullable=False
    )

    @declared_attr
    def track(cls) -> Mapped["Track"]:
        return relationship(back_populates="track_playlist_associations")


class PlaylistTrackAssocSync(CustomSyncBase, PlaylistTrackAssocMixin):
    pass


class PlaylistTrackAssoc(CustomBase, PlaylistTrackAssocMixin):
    pass


class PlaylistCategoryAssocMixin(TimeStampMixin):
    playlist_id = mapped_column(ForeignKey("playlist.id"), primary_key=True, nullable=False)
    category_id: Mapped[int] = mapped_column(
        ForeignKey("category.id"), primary_key=True, nullable=False
    )

    @declared_attr
    def category(cls) -> Mapped["Category"]:
        return relationship(back_populates="category_playlist_associations")


class PlaylistCategoryAssocSync(CustomSyncBase, PlaylistCategoryAssocMixin):
    pass


class PlaylistCategoryAssoc(CustomBase, PlaylistCategoryAssocMixin):
    pass


class PlaylistCounterMixin:
    playlist_id: Mapped[int] = mapped_column(
        ForeignKey("playlist.id"), primary_key=True, nullable=False
    )
    play_count: Mapped[int] = mapped_column(default=0)
    schedule_count: Mapped[int] = mapped_column(default=0)


class PlaylistCounterSync(CustomSyncBase, PlaylistCounterMixin):
    pass


class PlaylistCounter(CustomBase, PlaylistCounterMixin):
    pass



import re
from datetime import datetime
from typing import Any

from sqlalchemy.orm import (
    DeclarativeBase,
    Mapped,
    declared_attr,
    mapped_column,
    object_session,
    relationship,
)
from sqlalchemy.ext.asyncio import AsyncAttrs
from sqlalchemy import Column, Uuid, event, inspect, TypeDecorator
from sqlalchemy.dialects.postgresql import TSVECTOR


def resolve_table_name(name: str) -> str:
    """Resolves table names to their mapped names."""
    names = re.split("(?=[A-Z])", name)  # noqa
    return "_".join([x.lower() for x in names if x])


class CustomBase(DeclarativeBase, AsyncAttrs):
    __repr_attrs__: list[Any] = []
    __repr_max_length__ = 15

    @declared_attr
    def __tablename__(self) -> str:
        return resolve_table_name(self.__name__)

    def dict(self) -> dict:
        """Returns a dict representation of a model."""
        return {c.name: getattr(self, c.name) for c in self.__table__.columns}

    @property
    def _id_str(self) -> str:
        ids = inspect(self).identity
        if ids:
            return "-".join([str(x) for x in ids]) if len(ids) > 1 else str(ids[0])
        else:
            return "None"

    @property
    def _repr_attrs_str(self) -> str:
        max_length = self.__repr_max_length__

        values = []
        single = len(self.__repr_attrs__) == 1
        for key in self.__repr_attrs__:
            if not hasattr(self, key):
                raise KeyError(
                    "{} has incorrect attribute '{}' in " "__repr__attrs__".format(
                        self.__class__, key
                    )
                )
            value = getattr(self, key)
            wrap_in_quote = isinstance(value, str)

            value = str(value)
            if len(value) > max_length:
                value = value[:max_length] + "..."

            if wrap_in_quote:
                value = "'{}'".format(value)
            values.append(value if single else "{}:{}".format(key, value))

        return " ".join(values)

    def __repr__(self) -> str:
        # get id like '#123'
        id_str = ("#" + self._id_str) if self._id_str else ""
        # join class name, id and repr_attrs
        return "<{} {}{}>".format(
            self.__class__.__name__,
            id_str,
            " " + self._repr_attrs_str if self._repr_attrs_str else "",
        )


class AuditMixin(object):
    created_by = Column(Uuid, nullable=True)
    updated_by = Column(Uuid, nullable=True)

    @declared_attr
    def created_by_user(cls):
        return relationship(
            "User",
            foreign_keys=[cls.created_by],
            primaryjoin="User.id==%s.created_by" % cls.__name__,
        )

    @declared_attr
    def updated_by_user(cls):
        return relationship(
            "User",
            foreign_keys=[cls.updated_by],
            primaryjoin="User.id==%s.updated_by" % cls.__name__,
        )

    @staticmethod
    def _updated_info(mapper: Any, connection: Any, target: Any) -> None:
        s = object_session(target)
        target.updated_at = datetime.utcnow()
        if user_id := getattr(s, "user_id", None):
            target.updated_by = user_id

    @staticmethod
    def _created_info(mapper: Any, connection: Any, target: Any) -> None:
        s = object_session(target)
        if user_id := getattr(s, "user_id", None):
            target.created_by = user_id

    @classmethod
    def get_session_user_id(cls, connection):
        return connection.info.get("user_id")

    @classmethod
    def __declare_last__(cls) -> None:
        event.listen(cls, "before_insert", cls._created_info)
        event.listen(cls, "before_update", cls._updated_info)


class TimeStampMixin(object):
    """Timestamping mixin"""

    created_at: Mapped[datetime] = mapped_column(default=datetime.utcnow)
    created_at._creation_order = 9998
    updated_at: Mapped[datetime] = mapped_column(default=datetime.utcnow)
    updated_at._creation_order = 9998

    @staticmethod
    def _updated_at(mapper, connection, target):
        target.updated_at = datetime.utcnow()

    @classmethod
    def __declare_last__(cls):
        event.listen(cls, "before_update", cls._updated_at)


# class InheritanceMixin(object):
#     @declared_attr
#     def __mapper_args__(cls):
#         return {
#             'polymorphic_identity': cls.__name__.lower(),
#         }


# class CustomBase(Base, AsyncAttrs):
#     pass


# class SyncInheritanceMixin(object):
#     type = Column(String(50))

#     @declared_attr
#     def __mapper_args__(cls):
#         return {
#             'polymorphic_identity': cls.__name__.lower(),
#             'polymorphic_on': 'type',
#         }


class CustomSyncBase(DeclarativeBase):
    __repr_attrs__: list[Any] = []
    __repr_max_length__ = 15
    __abstract__ = True

    @declared_attr
    def __tablename__(self) -> str:
        return resolve_table_name(self.__name__)

    """ Is this missed here or not moved intentionally?"""

    # def serializable_dict(self) -> Dict[str, Any]:
    #     d = {col.name: getattr(self, col.name) for col in self.__table__.columns}
    #     return orjson.loads(orjson.dumps(jsonable_encoder(d)))

    def dict(self) -> dict:
        """Returns a dict representation of a model."""
        return {c.name: getattr(self, c.name) for c in self.__table__.columns}

    @property
    def _id_str(self) -> str:
        ids = inspect(self).identity
        if ids:
            return "-".join([str(x) for x in ids]) if len(ids) > 1 else str(ids[0])
        else:
            return "None"

    @property
    def _repr_attrs_str(self) -> str:
        max_length = self.__repr_max_length__

        values = []
        single = len(self.__repr_attrs__) == 1
        for key in self.__repr_attrs__:
            if not hasattr(self, key):
                raise KeyError(
                    "{} has incorrect attribute '{}' in " "__repr__attrs__".format(
                        self.__class__, key
                    )
                )
            value = getattr(self, key)
            wrap_in_quote = isinstance(value, str)

            value = str(value)
            if len(value) > max_length:
                value = value[:max_length] + "..."

            if wrap_in_quote:
                value = "'{}'".format(value)
            values.append(value if single else "{}:{}".format(key, value))

        return " ".join(values)

    def __repr__(self) -> str:
        # get id like '#123'
        id_str = ("#" + self._id_str) if self._id_str else ""
        # join class name, id and repr_attrs
        return "<{} {}{}>".format(
            self.__class__.__name__,
            id_str,
            " " + self._repr_attrs_str if self._repr_attrs_str else "",
        )


class TSVector(TypeDecorator):
    impl = TSVECTOR



import uuid
from typing import List

from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship

from artemis_model.base import TimeStampMixin, CustomSyncBase, CustomBase

from sqlalchemy.ext.declarative import declared_attr


class PermissionMixin(TimeStampMixin):
    id: Mapped[str] = mapped_column(primary_key=True)
    name: Mapped[str] = mapped_column(nullable=False)
    description: Mapped[str] = mapped_column(nullable=False)
    type: Mapped[str] = mapped_column(
        nullable=False
    )  # music-management, player-management, user-management

    @declared_attr
    def user_permission_associations(cls) -> Mapped[List["UserPermissionAssoc"]]:
        return relationship(back_populates="permission", cascade="all, delete-orphan")


class UserPermissionAssocMixin(TimeStampMixin):
    user_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("user.id"), primary_key=True)
    permission_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("permission.id"), primary_key=True)

    @declared_attr
    def permission(cls) -> Mapped["Permission"]:
        return relationship(back_populates="user_permission_associations")


class UserPermissionAssocSync(CustomSyncBase, UserPermissionAssocMixin):
    pass


class UserPermissionAssoc(CustomBase, UserPermissionAssocMixin):
    pass


class PermissionSync(CustomSyncBase, PermissionMixin):
    pass


class Permission(CustomBase, PermissionMixin):
    pass



from datetime import datetime
from typing import Annotated, NotRequired
from typing_extensions import TypedDict
import uuid

from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy.types import JSON, UUID
from pydantic import Field, RootModel, BaseModel

from artemis_model.base import CustomBase, TimeStampMixin, AuditMixin, CustomSyncBase

from sqlalchemy.ext.declarative import declared_attr


class ScheduleMixin(TimeStampMixin, AuditMixin):
    """
    This table is used to store the schedule of a zone.
    """

    id: Mapped[int] = mapped_column(autoincrement=True, primary_key=True, index=True)
    name: Mapped[str] = mapped_column(nullable=False)
    zone_id: Mapped[int] = mapped_column(ForeignKey("zone.id"), index=True)
    is_active: Mapped[bool] = mapped_column(default=True, nullable=False)
    timesheet: Mapped[JSON] = mapped_column(JSON, nullable=True)
    locked_by: Mapped[str] = mapped_column(default=None, nullable=True)
    legacy_id: Mapped[uuid.UUID] = mapped_column(UUID(as_uuid=True), nullable=True)

    @declared_attr
    def zone(cls) -> Mapped["Zone"]:
        return relationship(back_populates="schedules")


class ScheduleSync(CustomSyncBase, ScheduleMixin):
    pass


class Schedule(CustomBase, ScheduleMixin):
    pass


class SchedulePresetMixin(TimeStampMixin, AuditMixin):
    """
    This table is used to store preset schedules regarding location type.
    """

    id: Mapped[int] = mapped_column(autoincrement=True, primary_key=True, index=True)
    type: Mapped[str] = mapped_column(nullable=False)
    is_active: Mapped[bool] = mapped_column(default=True, nullable=False)
    timesheet: Mapped[JSON] = mapped_column(JSON, nullable=True)


class SchedulePresetSync(CustomSyncBase, SchedulePresetMixin):
    pass


class SchedulePreset(CustomBase, SchedulePresetMixin):
    pass


class TimeslotData(TypedDict):
    """Timeslot data schema."""

    p: NotRequired[list[int]]
    d: NotRequired[list[int]]
    scheduled_at: NotRequired[datetime]


class Timesheet(RootModel):
    """Timesheet schema."""

    root: dict[Annotated[str, Field(pattern=r"^\d{2}:\d{2}$")], TimeslotData]


class ScheduleItem(BaseModel):
    """Schedule item schema."""

    zone_id: int
    timesheet: Timesheet


__all__ = [
    "ScheduleItem",
    "Timesheet",
    "TimeslotData",
    "SchedulePreset",
    "SchedulePresetSync",
    "ScheduleSync",
    "Schedule",
]
